function Game() {
    this.$betText = $("#bet_text");
    this.$moneyText = $("#money_text");
    this.$betOneBtn = $("#bet_1_btn");
    this.$betThreeBtn = $("#bet_3_btn");
    this.$betFiveBtn = $("#bet_5_btn");
    this.$bet50Btn = $("#bet_50_btn");
    this.$bet100Btn = $("#bet_100_btn");
    this.$bet1000Btn = $("#bet_1000_btn");
    this.$bet5000Btn = $("#bet_5000_btn");
    this.$betAllBtn = $("#bet_all_btn");
    this.$startBtn = $("#start_btn");
    this.$reset=$("#reset");
    this.$endGameBtn = $("#save_result_btn");

    this.$endRoundScreen = $("#end_round_screen");
    this.$typeWinText = $("#type_win_text");
    this.$comboText = $("#combo_text");
    this.$moneyWonText = $("#money_won_text");

    this.currentBet = this.currentMultiplier = this.winText = this.combo = this.moneyWon = undefined;
    this.slots = this.slot1 = this.slot2 = this.slot3 = undefined;
    this.rotating = false;
    this.tempRandomSlots = undefined;
    this.rotatingSlots = this.spinings = this.stoppedSlots = 0;
    this.rotatingTimeOut = undefined;
    this.setSlots();
    this.setListeners();
}

Game.prototype.setSlots = function() {
    this.slots = new Array();
    this.slot1 = new Slot($("#inner_slots_1"), $("#slot_1"));
    this.slot2 = new Slot($("#inner_slots_2"), $("#slot_2"));
    this.slot3 = new Slot($("#inner_slots_3"), $("#slot_3"));
    this.slots.push(this.slot1, this.slot2, this.slot3);
    this.setSlotListener(this.slot1);
    this.setSlotListener(this.slot2);
    this.setSlotListener(this.slot3);
}

Game.prototype.setSlotListener = function(slot) {
    $(slot).bind(variables.SLOT_STOPPED_EVENT, function(e) {
        objects.game.rotatingSlots++;
        if(objects.game.rotatingSlots == objects.game.slots.length) {
            objects.game.spinings++;
            if(objects.game.spinings < settings.SPININGS_BEFORE_STOP) {
                objects.game.rotateSlots();
            }else {
                objects.game.setAllSlotsToPlace();
            }
        }
    });

    $(slot).bind(variables.SLOT_ROTATION_ENDED, function(e) {
        objects.game.stoppedSlots++;
        if(objects.game.stoppedSlots == objects.game.slots.length) {
            objects.game.prepareToNextRound();
        }
    });
}

Game.prototype.prepareToNextRound = function() {
    this.rotatingSlots = this.spinings = this.stoppedSlots = 0;
    this.rotating = false;
    this.setRoundMoney();
    this.setBetText();
    this.setCurrentMoneyText();
    this.setEndRoundScreen();
    if(this.currentMultiplier > 0) this.combo++;
}

Game.prototype.setAllSlotsToPlace = function() {
    this.setSlotsTargetIndex();
    for(var i = 0; i < this.slots.length; i++) {
        this.slots[i].setToTargetPlace();
    }
}

Game.prototype.setSlotsTargetIndex = function() {
    this.tempRandomSlots = new Array();
    this.tempRandomSlots.push(this.slot1, this.slot2, this.slot3);
    var tempSlot1 = this.getRandomSlot();
    var tempSlot2 = this.getRandomSlot();
    var tempSlot3 = this.getRandomSlot();
    switch (this.winText) {
        case variables.THREE_OF_TYPE_TEXT :
            tempSlot1.targetIcon = utils.getRandomIcon();
            tempSlot2.targetIcon = utils.getSameTypeButDifferentIcon(tempSlot1.targetIcon);
            tempSlot3.targetIcon = utils.getSameTypeIcon(tempSlot1.targetIcon);
            tempSlot1.needToShine = true;
            tempSlot2.needToShine = true;
            tempSlot3.needToShine = true;
            break;
        case variables.TWO_SAME_TEXT :
            tempSlot1.targetIcon = utils.getRandomIcon();
            tempSlot2.targetIcon = tempSlot1.targetIcon;
            tempSlot3.targetIcon = utils.getDifferentIcon(tempSlot1.targetIcon);
            tempSlot1.needToShine = true;
            tempSlot2.needToShine = true;
            break;
        case  variables.THREE_SAME_TEXT :
            tempSlot1.targetIcon = utils.getRandomIcon();
            tempSlot2.targetIcon = tempSlot1.targetIcon;
            tempSlot3.targetIcon = tempSlot1.targetIcon;
            tempSlot1.needToShine = true;
            tempSlot2.needToShine = true;
            tempSlot3.needToShine = true;
            break;
        default :
            tempSlot1.targetIcon = utils.getRandomIcon();
            tempSlot2.targetIcon = utils.getDifferentIcon(tempSlot1.targetIcon);
            tempSlot3.targetIcon = utils.getDifferentOfTwoIcon(tempSlot1.targetIcon, tempSlot2.targetIcon);
            break
    }

}

Game.prototype.getRandomSlot = function() {
    var index = Math.round(Math.random() * (this.tempRandomSlots.length - 1));
    var tempSlot = this.tempRandomSlots[index];
    this.tempRandomSlots.splice(index, 1);
    return tempSlot;
}

Game.prototype.setListeners = function() {
    this.$betOneBtn.click(function(e) {
        if (objects.game.rotating == true) return;
        objects.game.setBet(1);
    });
    this.$betThreeBtn.click(function(e) {
        if (objects.game.rotating == true) return;
        objects.game.setBet(3);
    });
    this.$betFiveBtn.click(function(e) {
        if (objects.game.rotating == true) return;
        objects.game.setBet(5);
    });
    this.$bet50Btn.click(function(e) {
        if (objects.game.rotating == true) return;
        objects.game.setBet(50);
    });
    this.$bet100Btn.click(function(e) {
        if (objects.game.rotating == true) return;
        objects.game.setBet(100);
    });
    this.$bet1000Btn.click(function(e) {
        if (objects.game.rotating == true) return;
        objects.game.setBet(1000);
    });
    this.$bet5000Btn.click(function(e) {
        if (objects.game.rotating == true) return;
        objects.game.setBet(5000);
    });
    this.$reset.click(function (e) {
        if (objects.game.rotating == true) return;
        objects.game.reset();
    });
    this.$betAllBtn.click(function(e) {
        if (objects.game.rotating == true) return;
        objects.game.setBet(variables.currentMoney);
    });
    this.$endGameBtn.click(function(e){
        if (objects.game.rotating == true) return;
        objects.game.endGame();
    });
    this.$startBtn.click(function(e) {
        if (objects.game.rotating == true) return;
        objects.game.startRound();
    });
    this.$endRoundScreen.click(function (e) {
        objects.game.$endRoundScreen.addClass(domElements.invisibleClassName);
        if(variables.currentMoney <= 0) objects.game.endGame();
    });
}

Game.prototype.endGame = function () {
    objects.records.setNewRecords();
    utils.turnOnScreen(domElements.$recordScreen);
}

Game.prototype.startRound = function () {
    if(this.currentBet == 0) {
//        alert("set your bet please!");
        return;
    }
    this.setCurrentMultiplier();
    this.spinings = 0;
    this.rotateSlots();
}

Game.prototype.rotateSlots = function() {
    this.rotating = true;
    this.rotatingSlots = 0;
    for(var i = 0; i < this.slots.length; i++) {
        this.slots[i].rotate();
    }
}

Game.prototype.setRoundMoney = function() {
    this.moneyWon = this.currentBet * this.currentMultiplier * this.combo;
    variables.currentMoney = this.moneyWon == 0 ? variables.currentMoney - this.currentBet : variables.currentMoney + this.moneyWon;
    this.currentBet = 0;
}

Game.prototype.setEndRoundScreen = function() {
    this.$endRoundScreen.removeClass(domElements.invisibleClassName);
    this.$typeWinText.text(this.winText);
    this.$comboText.text("");
    if(this.combo > 1) this.$comboText.text("Combo x" + this.combo);
    this.$moneyWonText.text(this.moneyWon);
}

Game.prototype.setCurrentMultiplier = function() {
    var globalRnd = Math.random();
    if (globalRnd > settings.LUCK_PERCENT / 100) {
        this.currentMultiplier = 0;
        this.winText = variables.NO_WIN_TEXT;
        $("#money_won_text0").addClass("invisible");
        $("#money_won_text").addClass("invisible");
        this.combo = 1;
        return;
    }
    var rnd = Math.random();
    if (rnd < 0.33) {
        this.currentMultiplier = settings.TWO_SAME_FACTOR;
        this.winText = variables.TWO_SAME_TEXT;
        $("#money_won_text0").removeClass("invisible");
        $("#money_won_text").removeClass("invisible");
    }else if (rnd >= 0.33 && rnd <= 0.66) {
        this.currentMultiplier = settings.THREE_SAME_FACTOR;
        this.winText = variables.THREE_SAME_TEXT;
        $("#money_won_text0").removeClass("invisible");
        $("#money_won_text").removeClass("invisible");
    }else {
        this.currentMultiplier = settings.THREE_OF_TYPE_FACTOR;
        this.winText = variables.THREE_OF_TYPE_TEXT;
        $("#money_won_text0").removeClass("invisible");
        $("#money_won_text").removeClass("invisible");
    }
}

Game.prototype.reset = function() {
        this.currentBet = 0;
        this.setBetText();
}


Game.prototype.setBet = function(newBet) {

    if (newBet < 1 ) newBet = 0;
        this.currentBet = this.currentBet+newBet;
        if (this.currentBet > variables.currentMoney) this.currentBet = variables.currentMoney;
        this.setBetText();
}

Game.prototype.startNewGame = function() {
    this.currentBet = 0;
    this.combo = 1;
    variables.currentMoney = settings.START_MONEY;
    this.setBetText();
    this.setCurrentMoneyText();
}

Game.prototype.setBetText = function() {
    this.$betText.text(this.currentBet);
}

Game.prototype.setCurrentMoneyText = function() {
    this.$moneyText.text(variables.currentMoney);
}